@@ -25,12 +25,14 @@ from typing import (
     Type,
     Union,
     cast,
+    overload,
 )
 
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 from tqdm.auto import tqdm
+from typing_extensions import Literal
 
 from ...typing import Array, ArrayLike, Axes, Axis, Figure
 from ...utils.plotting import _get_plot_kwargs, _style_subplot, plot, plot_grid
@@ -130,6 +132,20 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
                 self.time.append(time)
                 self.results.append(results)
 
+    @overload
+    def get_time(self, concatenated: Literal[False]) -> Union[None, List[Array]]:  # type: ignore
+        # The type error here comes from List[Array] and Array overlapping
+        # Can safely be ignore
+        ...
+
+    @overload
+    def get_time(self, concatenated: Literal[True]) -> Union[None, Array]:
+        ...
+
+    @overload
+    def get_time(self, concatenated: bool = True) -> Union[None, Array]:
+        ...
+
     def get_time(self, concatenated: bool = True) -> Union[None, Array, List[Array]]:
         """Get simulation time.
 
@@ -217,8 +233,27 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
         self._save_simulation_results(time=time_array, results=results_array, skipfirst=False)
         return time_array, results_array
 
+    @overload
+    def get_results_array(  # type: ignore
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Union[None, List[Array]]:
+        ...
+
+    @overload
     def get_results_array(
         self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Union[None, Array]:
+        ...
+
+    def get_results_array(
+        self,
+        *,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
     ) -> Union[None, Array, List[Array]]:
@@ -233,23 +268,81 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
             return np.concatenate(results, axis=0)  # type: ignore
         return results
 
+    @overload
+    def get_results_dict(  # type: ignore
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Union[None, List[Dict[str, Array]]]:
+        ...
+
+    @overload
     def get_results_dict(
         self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Union[None, Dict[str, Array]]:
+        ...
+
+    @overload
+    def get_results_dict(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Union[None, Dict[str, Array]]:
+        ...
+
+    def get_results_dict(
+        self,
+        *,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
     ) -> Union[None, Dict[str, Array], List[Dict[str, Array]]]:
         """Get simulation results."""
-        results = self.get_results_array(normalise=normalise, concatenated=concatenated)
-        if results is None:
-            return None
         if concatenated:
-            results = cast(Array, results)  # this is just a typing annotation
+            results = self.get_results_array(normalise=normalise, concatenated=True)
+            if results is None:
+                return None
             return dict(zip(self.model.get_compounds(), results.T))
-        results = cast(List[Array], results)  # this is just a typing annotation
-        return [dict(zip(self.model.get_compounds(), i.T)) for i in results]
+        else:
+            results_ = self.get_results_array(normalise=normalise, concatenated=False)
+            if results_ is None:
+                return None
+            return [dict(zip(self.model.get_compounds(), i.T)) for i in results_]
+
+    @overload
+    def get_results_df(  # type: ignore
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Union[None, List[pd.DataFrame]]:
+        ...
+
+    @overload
+    def get_results_df(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Union[None, pd.DataFrame]:
+        ...
+
+    @overload
+    def get_results_df(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Union[None, pd.DataFrame]:
+        ...
 
     def get_results_df(
         self,
+        *,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
     ) -> Union[None, pd.DataFrame, List[pd.DataFrame]]:
@@ -474,22 +567,49 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         return time, results
 
     def _calculate_fluxes(self) -> None:
-        if any((i is None for i in [self.time, self.results, self.simulation_parameters])):
-            pass
-        else:
-            fluxes = []
-            for t, y, pars in zip(
-                cast(List[Array], self.time),
-                cast(List[Array], self.results),
-                cast(List[Dict[str, float]], self.simulation_parameters),
-            ):
-                self.update_parameters(parameters=pars)
-                fluxes_array = self.model.get_fluxes_array(y=y, t=t)
-                fluxes.append(fluxes_array)
-            self.fluxes = fluxes
+        time = self.time
+        results = self.results
+        pars = self.simulation_parameters
+        if time is None or results is None or pars is None:
+            return None
+
+        fluxes = []
+        for t, y, p in zip(time, results, pars):
+            self.update_parameters(parameters=p)
+            fluxes_array = self.model.get_fluxes_array(y=y, t=t)
+            fluxes.append(fluxes_array)
+        self.fluxes = fluxes
+
+    @overload
+    def get_fluxes_array(  # type: ignore
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Optional[List[Array]]:
+        ...
+
+    @overload
+    def get_fluxes_array(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Optional[Array]:
+        ...
+
+    @overload
+    def get_fluxes_array(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Optional[Array]:
+        ...
 
     def get_fluxes_array(
         self,
+        *,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
     ) -> Optional[Union[Array, List[Array]]]:
@@ -498,6 +618,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return None
         if self.fluxes is None:
             self._calculate_fluxes()
+        # Cast is ok
         fluxes = cast(List[Array], self.fluxes)
         if normalise is not None:
             fluxes = self._normalise_split_array(split_array=fluxes, normalise=normalise)
@@ -505,8 +626,36 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return np.concatenate(fluxes, axis=0)  # type: ignore
         return fluxes
 
+    @overload
+    def get_fluxes_dict(  # type: ignore
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Optional[List[Dict[str, Array]]]:
+        ...
+
+    @overload
     def get_fluxes_dict(
         self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Optional[Dict[str, Array]]:
+        ...
+
+    @overload
+    def get_fluxes_dict(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Optional[Dict[str, Array]]:
+        ...
+
+    def get_fluxes_dict(
+        self,
+        *,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
     ) -> Optional[Union[Dict[str, Array], List[Dict[str, Array]]]]:
@@ -518,8 +667,36 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return dict(zip(self.model.rates, cast(Array, fluxes).T))
         return [dict(zip(self.model.rates, i.T)) for i in fluxes]
 
+    @overload
     def get_fluxes_df(
         self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Optional[List[pd.DataFrame]]:
+        ...
+
+    @overload
+    def get_fluxes_df(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Optional[pd.DataFrame]:
+        ...
+
+    @overload
+    def get_fluxes_df(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Optional[pd.DataFrame]:
+        ...
+
+    def get_fluxes_df(
+        self,
+        *,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
     ) -> Optional[Union[pd.DataFrame, List[pd.DataFrame]]]:
@@ -531,7 +708,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         if concatenated:
             return pd.DataFrame(
                 data=fluxes,
-                index=self.get_time(),
+                index=time,
                 columns=self.model.get_rate_names(),
             )
         return [
@@ -552,8 +729,36 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             full_results.append(np.reshape(list(results.values()), (len(results), len(t))).T)  # type: ignore
         self.full_results = full_results
 
+    @overload
+    def get_full_results_array(  # type: ignore
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Optional[List[Array]]:
+        ...
+
+    @overload
     def get_full_results_array(
         self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Optional[Array]:
+        ...
+
+    @overload
+    def get_full_results_array(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Optional[Array]:
+        ...
+
+    def get_full_results_array(
+        self,
+        *,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
     ) -> Optional[Union[Array, List[Array]]]:
@@ -567,6 +772,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return None
         if self.full_results is None:
             self._calculate_full_results()
+        # Cast is ok
         full_results = cast(List[Array], self.full_results).copy()
         if normalise is not None:
             full_results = self._normalise_split_array(
@@ -577,8 +783,36 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return np.concatenate(full_results, axis=0)  # type: ignore
         return full_results
 
+    @overload
+    def get_full_results_dict(  # type: ignore
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Optional[List[Dict[str, Array]]]:
+        ...
+
+    @overload
     def get_full_results_dict(
         self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Optional[Dict[str, Array]]:
+        ...
+
+    @overload
+    def get_full_results_dict(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Optional[Dict[str, Array]]:
+        ...
+
+    def get_full_results_dict(
+        self,
+        *,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
     ) -> Optional[Union[Dict[str, Array], List[Dict[str, Array]]]]:
@@ -591,8 +825,36 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return dict(zip(all_compounds, cast(Array, full_results).T))
         return [dict(zip(all_compounds, i.T)) for i in full_results]
 
+    @overload
     def get_full_results_df(
         self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Optional[List[pd.DataFrame]]:
+        ...
+
+    @overload
+    def get_full_results_df(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Optional[pd.DataFrame]:
+        ...
+
+    @overload
+    def get_full_results_df(
+        self,
+        *,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Optional[pd.DataFrame]:
+        ...
+
+    def get_full_results_df(
+        self,
+        *,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
     ) -> Optional[Union[pd.DataFrame, List[pd.DataFrame]]]:
@@ -607,8 +869,39 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return pd.DataFrame(data=full_results, index=time, columns=all_compounds)
         return [pd.DataFrame(data=res, index=t, columns=all_compounds) for t, res in zip(time, full_results)]
 
+    @overload
+    def get_variable(  # type: ignore
+        self,
+        *,
+        variable: str,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> Optional[List[Array]]:
+        ...
+
+    @overload
     def get_variable(
         self,
+        *,
+        variable: str,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Optional[Array]:
+        ...
+
+    @overload
+    def get_variable(
+        self,
+        *,
+        variable: str,
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Optional[Array]:
+        ...
+
+    def get_variable(
+        self,
+        *,
         variable: str,
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
@@ -626,8 +919,39 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return cast(Dict[str, Array], full_results_dict)[variable]
         return [i[variable] for i in cast(List[Dict[str, Array]], full_results_dict)]
 
+    @overload
+    def get_variables(  # type: ignore
+        self,
+        *,
+        variables: List[str],
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[False],
+    ) -> List[Array]:
+        ...
+
+    @overload
     def get_variables(
         self,
+        *,
+        variables: List[str],
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: Literal[True],
+    ) -> Array:
+        ...
+
+    @overload
+    def get_variables(
+        self,
+        *,
+        variables: List[str],
+        normalise: Union[float, ArrayLike] | None = None,
+        concatenated: bool = True,
+    ) -> Array:
+        ...
+
+    def get_variables(
+        self,
+        *,
         variables: List[str],
         normalise: Union[float, ArrayLike] | None = None,
         concatenated: bool = True,
